package org.springframework.boot.loader.jar;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.springframework.boot.loader.data.RandomAccessData;
import org.springframework.boot.loader.jar.CentralDirectoryEndRecord;
import org.springframework.boot.loader.jar.CentralDirectoryFileHeader;
import org.springframework.boot.loader.jar.CentralDirectoryVisitor;

class CentralDirectoryParser {
  private static final int CENTRAL_DIRECTORY_HEADER_BASE_SIZE = 46;
  
  private final List<CentralDirectoryVisitor> visitors = new ArrayList<>();
  
  <T extends CentralDirectoryVisitor> T addVisitor(T visitor) {
    this.visitors.add((CentralDirectoryVisitor)visitor);
    return visitor;
  }
  
  RandomAccessData parse(RandomAccessData data, boolean skipPrefixBytes) throws IOException {
    CentralDirectoryEndRecord endRecord = new CentralDirectoryEndRecord(data);
    if (skipPrefixBytes)
      data = getArchiveData(endRecord, data); 
    RandomAccessData centralDirectoryData = endRecord.getCentralDirectory(data);
    visitStart(endRecord, centralDirectoryData);
    parseEntries(endRecord, centralDirectoryData);
    visitEnd();
    return data;
  }
  
  private void parseEntries(CentralDirectoryEndRecord endRecord, RandomAccessData centralDirectoryData) throws IOException {
    byte[] bytes = centralDirectoryData.read(0L, centralDirectoryData.getSize());
    CentralDirectoryFileHeader fileHeader = new CentralDirectoryFileHeader();
    int dataOffset = 0;
    for (int i = 0; i < endRecord.getNumberOfRecords(); i++) {
      fileHeader.load(bytes, dataOffset, null, 0L, null);
      visitFileHeader(dataOffset, fileHeader);
      dataOffset += 46 + fileHeader.getName().length() + fileHeader
        .getComment().length() + (fileHeader.getExtra()).length;
    } 
  }
  
  private RandomAccessData getArchiveData(CentralDirectoryEndRecord endRecord, RandomAccessData data) {
    long offset = endRecord.getStartOfArchive(data);
    if (offset == 0L)
      return data; 
    return data.getSubsection(offset, data.getSize() - offset);
  }
  
  private void visitStart(CentralDirectoryEndRecord endRecord, RandomAccessData centralDirectoryData) {
    for (CentralDirectoryVisitor visitor : this.visitors)
      visitor.visitStart(endRecord, centralDirectoryData); 
  }
  
  private void visitFileHeader(long dataOffset, CentralDirectoryFileHeader fileHeader) {
    for (CentralDirectoryVisitor visitor : this.visitors)
      visitor.visitFileHeader(fileHeader, dataOffset); 
  }
  
  private void visitEnd() {
    for (CentralDirectoryVisitor visitor : this.visitors)
      visitor.visitEnd(); 
  }
}
